/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <config.h>

#include <string.h>

#include <glib.h>
#include <gtk/gtk.h>

#include <pan/base/pan-i18n.h>
#include <pan/dialogs/dialogs.h>
#include <pan/dialogs/pan-about.h>

static char * get_credits_code (void);
static char * get_credits_users (void);
static char * get_credits_translators (void);

/**
 * dialog_about:
 *
 * About window to display program information, version number, and
 * copyright.
 **/
void
dialog_about (GtkWidget * window)
{
	char * pch;
	char * credits_bugs;
	char * credits_i18n;
	char * credits_code;
	GtkWidget * about;

	about = pan_about_new (GTK_WINDOW(window),
	                       "Pan", VERSION,
	                       _("A newsreader for GNOME"),
	                       _("Copyright (C) 2002 Charles Kerr"),
	                       PAN_ABOUT_LICENSE_GPL,
	                       "http://pan.rebelbase.com/");

	pan_about_add_author (about, "Charles Kerr", "charles@rebelbase.com",
	                      _("Programmer"));
	pan_about_add_author (about, "Christophe Lambin", "clambin@easynet.be",
	                      _("Programmer"));
	pan_about_add_author (about, "Adam Huffman", "bloch@verdurin.com",
	                      _("Programmer & Documentation"));
	pan_about_add_author (about, "Matt Eagleson", "e.Messiah@superpimp.org",
	                      _("Project Originator, Programmer"));
	pan_about_add_author (about, "Jason Leach", "jleach@ximian.com",
	                      _("Programmer, 0.6 - 0.7"));

	credits_i18n = get_credits_translators ();
	credits_code = get_credits_code ();
	credits_bugs = get_credits_users ();
	pch = g_strdup_printf (_("Pan %s Translations\n%s\n"
                                 "\n"
                                 "Pan %s Code or Patches\n%s\n"
                                 "\n"
                                 "Pan %s Suggestions and Testing\n%s\n"
                                 "\n"
	                         "Contributors to previous versions are listed at\n"
	                         "http://pan.rebelbase.com/CREDITS"),
	                         VERSION, credits_i18n,
	                         VERSION, credits_code,
	                         VERSION, credits_bugs);
	pan_about_add_text (about, _("Thanks"), pch);
	g_free (credits_i18n);
	g_free (credits_code);
	g_free (credits_bugs);
	g_free (pch);

	gtk_widget_show (about);
}


static char *
get_credits_translators (void)
{
	const char * str =
"\n\tcs - Michal Bukovjan"
"\n\tes - Pablo del Campo"
"\n\tet - Tivo Leedjrv"
"\n\tfr - Christophe Merlet"
"\n\thu - Andras Timar"
"\n\tko - Nam SungHyun"
"\n\tnl - Reinout van Schouwen"
"\n\tno - Tov Are Jacobsen"
"\n\tpl - GNOME PL Team"
"\n\tpt - Duarte Loreto"
"\n\tsk - Marcel Telka"
"\n\tsv - Christian Rose"
"\n\tvi - pclouds";
	return g_convert (str, strlen(str), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
}

static char *
get_credits_code (void)
{
	const char * str =
"\n\tMichael Alger"
"\n\tMark Derricutt"
"\n\tMartin Tomasek";
	return g_convert (str, strlen(str), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
}

static char *
get_credits_users (void)
{
	const char * str = 
"\n\tAlan (#88969)"
"\n\tLars Bamberger"
"\n\tPablo del Campo (#90115)"
"\n\tGabi Davar (#85445)"
"\n\tDevilKin (#88790, #88008, #88010, #89603, #89584)"
"\n\tAaron Digula (#88729)"
"\n\tLaurent Duperval (#87854, #88386)"
"\n\tFred Fraley (#86409)"
"\n\tBenjamin Greiner (#90036, #90041, #90045)"
"\n\tPhilippe Gramoull"
"\n\tHimanshu Gohel (#74892)"
"\n\tCarl Hudkins (#80002, #85725, #86215, #86795)"
"\n\tAlex Jaques (#88665)"
"\n\tjcnbulk (#85968)"
"\n\tKang Jeong-Hee (#81181)"
"\n\tWolfgang Karall (#79993)"
"\n\tKevin (#88921)"
"\n\tToivo Leedjarv (#87065)"
"\n\tD.m.D. Ljungmark (#87518)"
"\n\tChristian Lohmaier (#88532, #90068)"
"\n\tWilliam Maddler (#85510)"
"\n\tWilliam MacCracken (#89260)"
"\n\tBas Mevissen (#89455, #90491)"
"\n\tRoman Moeller (#82751)"
"\n\tBrian Morrison (#88255, #88257)"
"\n\tJohan Ovlinger (#90966)"
"\n\tPierre (#86203)"
"\n\tquimvy72 (#88027)"
"\n\tMike Robinson (#86881)"
"\n\tMarkus Schorpp (#88092)"
"\n\tSi (#72555)"
"\n\tAlexander Skwar (#85502, #85503, #85508)"
"\n\tSam Solon"
"\n\tMartin Tomasek (#84995, #88669, #85289, #88670, #89510)"
"\n\tAlen Williams (#88106)";
	return g_convert (str, strlen(str), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
}
